#include "FreeRTOS.h"
#include "task.h"

namespace osal {

class ErrorHandler {
   public:
	static void (*onAssertFailed)(const char* file, uint32_t line);
	static void (*onStackOverflow)(TaskHandle_t task, char* taskName);
};

}

void (*osal::ErrorHandler::onAssertFailed)(const char* file,
                                           uint32_t line) = NULL;

void (*osal::ErrorHandler::onStackOverflow)(TaskHandle_t task, char* taskName);

void assert_failed(const char* file, uint32_t line) {
	if (osal::ErrorHandler::onAssertFailed)
		osal::ErrorHandler::onAssertFailed(file, line);
}

void vApplicationGetIdleTaskMemory(StaticTask_t** ppxIdleTaskTCBBuffer,
                                   StackType_t** ppxIdleTaskStackBuffer,
                                   uint32_t* pulIdleTaskStackSize) {
	static StaticTask_t xIdleTaskTCB;
	static StackType_t uxIdleTaskStack[configMINIMAL_STACK_SIZE];

	*ppxIdleTaskTCBBuffer = &xIdleTaskTCB;
	*ppxIdleTaskStackBuffer = uxIdleTaskStack;
	*pulIdleTaskStackSize = configMINIMAL_STACK_SIZE;
}

void vApplicationStackOverflowHook(TaskHandle_t xTask, char* pcTaskName) {
	if (osal::ErrorHandler::onStackOverflow)
		osal::ErrorHandler::onStackOverflow(xTask, pcTaskName);
}
