#pragma once

/**
 * FreeRTOS Includes
 */
#include <FreeRTOS.h>
#include <task.h>
#include <itask.hpp>

/**
 * STL Includes
 */
#include <array>

namespace osal {

template <std::size_t NStackSize = configMINIMAL_STACK_SIZE>
class Task : public ITask {
   public:
	constexpr static uint32_t DEFAULT_PRIORITY = (configMAX_PRIORITIES - 1);

	Task(const char* name, uint32_t priority = DEFAULT_PRIORITY) : m_name(name), m_priority(priority) {}

	virtual ~Task() {}

	virtual void start() {
		xTaskCreateStatic(Task::runTask, this->m_name, this->stack.size(), (void*)this, this->m_priority,
		                  this->stack.data(), &this->buffer);
	}

	static void delayTick(int ticks) { vTaskDelay(ticks); }
	static void delayMs(int ms) { vTaskDelay(ms / portTICK_RATE_MS); }
	static void yield() { taskYIELD(); }

   private:
	static void runTask(void* pvParameters) { ((Task*)pvParameters)->run(); }

	std::array<StackType_t, NStackSize> stack;
	StaticTask_t buffer;

   protected:
	const char* m_name;
	const uint32_t m_priority;
};

}  // namespace osal
