#include <stdio.h>
#include <array>
#include <string>

#include <mutex.hpp>
#include <rtos.hpp>
#include <task.hpp>

#include <stm32f0xx.h>
#include <gpio.hpp>

using namespace osal;
using namespace stm32f072xb;
// https://www.st.com/resource/en/reference_manual/dm00031936-stm32f0x1stm32f0x2stm32f0x8-advanced-armbased-32bit-mcus-stmicroelectronics.pdf

constexpr uint32_t ledPin = 5;
ISemaphore* SemaphoreList[2];

void enableGpio() {
	// Gpio gpio = Gpio();
	SET_BIT(RCC->AHBENR, RCC_AHBENR_GPIOAEN);  // enable GPIOA (p.120)
	SET_BIT(RCC->AHBENR, RCC_AHBENR_GPIOCEN);  // enable GPIOC (p.120)
}

void setupLed(const uint32_t ledPin) {
	// LED

	GPIOA->MODER |= (0x1 << (ledPin * 2));    // Output mode (p.157)
	CLEAR_BIT(GPIOA->OTYPER, (1 << ledPin));  // Push-pull output (p.157)
}

void defaultAssert(const char* file, uint32_t line) {
	SET_BIT(GPIOA->ODR, 1 << ledPin);
}

class TaskA : public Task<> {
   public:
	TaskA() : Task("Task A") {}

	ISemaphore* ledSemaphore = SemaphoreList[0];

   protected:
	virtual void run() override {
		for (;;) {
			ledSemaphore->take();
			for (int i = 0; i < 2; i++) {
				SET_BIT(GPIOA->ODR, 1 << ledPin);
				Task::delayMs(500);
				CLEAR_BIT(GPIOA->ODR, 1 << ledPin);
				Task::delayMs(500);
			}
			ledSemaphore->give();
			Task::yield();
		}
	}
};

class TaskB : public Task<> {
   public:
	TaskB() : Task("Task B") {}

	ISemaphore* ledSemaphore = SemaphoreList[0];

   protected:
	virtual void run() override {
		for (;;) {
			ledSemaphore->take();
			for (int i = 0; i < 5; i++) {
				SET_BIT(GPIOA->ODR, 1 << ledPin);
				Task::delayMs(200);
				CLEAR_BIT(GPIOA->ODR, 1 << ledPin);
				Task::delayMs(200);
			}
			ledSemaphore->give();
			Task::yield();
		}
	}
};

class TaskC : public Task<> {
   public:
	TaskC() : Task("Task C") {}

	ISemaphore* ledSemaphore = SemaphoreList[0];

   protected:
	virtual void run() override {
		for (;;) {
			ledSemaphore->take();
			for (int i = 0; i < 20; i++) {
				SET_BIT(GPIOA->ODR, 1 << ledPin);
				Task::delayMs(50);
				CLEAR_BIT(GPIOA->ODR, 1 << ledPin);
				Task::delayMs(50);
			}
			ledSemaphore->give();
			Task::yield();
		}
	}
};

int main() {
	enableGpio();
	setupLed(ledPin);

	Mutex ledSemaphore = Mutex("Led Semaphore", false);
	SemaphoreList[0] = &ledSemaphore;

	TaskA taskA;
	taskA.start();

	TaskB taskB;
	taskB.start();

	TaskC taskC;
	taskC.start();

	Rtos::startScheduler();

	for (;;) {
	}

	return 0;
}
